//Using this array you can set your ratios for outputs A to H
unsigned long ratios[8] = {
  1,  //A
  2,  //B
  4,  //C
  8,  //D
  16, //E
  4,  //F
  8,  //G
  16  //H
};

#include <Adafruit_NeoPixel.h>
#include <elapsedMillis.h>
#include <PCF8574.h>
#include <Wire.h>
#include "variables.h"

PCF8574 expander;
Adafruit_NeoPixel LEDs = Adafruit_NeoPixel(10, A3, NEO_RGB + NEO_KHZ800);
elapsedMillis time_to_switch;
elapsedMicros outs_micros[8];

uint32_t blue = LEDs.Color(0, 25, 255);
uint32_t green = LEDs.Color(0, 255, 25);
uint32_t off = LEDs.Color(0, 0, 0);


void setup() {
  Serial.begin(9600);

  delay(100);

  LEDs.begin();
  LEDs.setBrightness(35);
  LEDs.show();

  //Setup GPIO & interrupts
  pinMode(TRIG, INPUT);
  pinMode(RESET, INPUT);

  for (int i = 0; i < 8; i++) {
    pinMode(outs[i], OUTPUT);
  }

  //I2C expander setup
  expander.begin(0x38);
  for (int i = 0; i < 8; i++) {
    expander.pinMode(i, INPUT_PULLUP);
  }

  //setting the I2C clock to 400k, the maximum for the PCF8574
  Wire.setClock(400000L);

  //I use INT0 and INT1 instead of the labels TRIG and RESET (as set up in variables.h)
  //because the Arduino IDE didn't translate those proprely to interrupts pin numbers
  attachInterrupt(INT0, trigger_int, CHANGE);
  attachInterrupt(INT1, reset_int, CHANGE);
}


void loop() {

  while (waiting_for_clock) {
    rainbowCycle(20);
  }

  for (int i = 0; i < 8; i++) {

    //------TURNS OFF OUTPUT THAT ONLY DIVIDES------////
    //rountine to turn off the necessary outputs
    //when the ouputs were on for the clock_length
    //we turn them off
    if (toggle_divide[i]) {
      if (out_state[i]) {
        if (micros() - time_turned_on[i] > period / (2 * ratios[i])) {
          FastWrite(i, LOW);
          out_state[i] = 0;
          LEDs.setPixelColor(i + 2, off);
        }
      }
    }

    //------TURNS on and off Multiplied outputs------////
    //this takes care of all the mutliple clocks between clocks pulses
    //turn them on and off
    if (!toggle_divide[i]) {
      //turns ons on the output when we reach a new division of the input clock
      if (!out_state[i]) {
        if (outs_micros[i] > period / ratios[i]) {
          outs_micros[i] = 0;
          FastWrite(i, HIGH);
          out_state[i] = 1;
          LEDs.setPixelColor(i + 2, green);
        }
      }

      //with this next line we reset the clock when we reach 50% cycle
      if (out_state[i]) {
        if (outs_micros[i] > period / (2 * ratios[i])) {
          FastWrite(i, LOW);
          out_state[i] = 0;
          LEDs.setPixelColor(i + 2, off);
        }
      }
    }
  }

  LEDs.show();

  //------Read switches------////
  //reading all switches takes 620 us
  if (time_to_switch > 350) {
    for (int i = 0; i < 8; i++) {
      toggle_divide[toggle_index[i]] = expander.digitalRead(i);
    }
    time_to_switch = 0;
  }

}
